/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.jmeter;

import java.io.FileInputStream;
import java.io.File;

import org.apache.jmeter.engine.StandardJMeterEngine;
import org.apache.jmeter.threads.ThreadGroup;
import org.xml.sax.InputSource;
import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.XMLReader;
import org.apache.jmeter.save.xml.XmlHandler;
import org.apache.jmeter.save.JMeterNameSpaceHandler;
import org.apache.jmeter.control.TestPlan;
import org.apache.jmeter.util.JMeterUtils;
import org.apache.jmeter.util.ListedHashTree;
import org.apache.jmeter.gui.tree.NonGuiTree;

import java.util.*;

/************************************************************
 *  Title: JMeter Description: Copyright: Copyright (c) 2000 Company: Apache
 *	This class uses a JMeter file that has been saved using org.apache.jmeter.NewDriver
 *	(The GUI version of the program). It eliminates the need for Swing components while
 *	running JMeter tests and hence can be run from non window clients.

 *  Usage:	java org.apache.jmeter.NonGuiDriver <propery file> <source file>
 *
 *	@see	org.apache.jmeter.gui.tree.NonGuiTree
 *
 *	@author     Tushar Bhatia
 *	@created    June 18, 2001
 *	@version    1.0
 ***********************************************************/

public class NonGuiDriver {

	public static void main(String arg[]){
		if(arg.length != 2){
			System.out.println("Non GUI Driver for JMeter Run. \nUsage:java org.apache.jmeter.NonGuiDriver <propery file> <source file>");
			System.exit(1);
		}
		JMeterUtils.getProperties(arg[0]);

		NonGuiDriver driver = new NonGuiDriver();
		driver.run(arg[1]);
	}

	private void run(String fileName){
		FileInputStream reader = null;
		try {
			 File f = new File(fileName);
			 if(!f.exists() || !f.isFile()) {
				 println("Could not open " + fileName);
				 return;
			 }

			reader = new FileInputStream(f);
			XmlHandler handler =
				 new XmlHandler(new JMeterNameSpaceHandler());
			XMLReader parser = JMeterUtils.getXMLParser();
			parser.setContentHandler(handler);
			parser.setErrorHandler(handler);
			parser.parse(new InputSource(reader));

			ListedHashTree tree = handler.getDataTree();
			println("Created the tree successfully");

			NonGuiTree ngt = new NonGuiTree();
			ngt.addSubTree(tree, null);
			TestPlan tp = (TestPlan)ngt.compileComponent(ngt.getChildAt(0));
			println("Compiled the Test Plan successfully");

			Collection groups = tp.getThreadGroups();
			StandardJMeterEngine engine = new StandardJMeterEngine();
			for (Iterator i = groups.iterator(); i.hasNext(); ) {
				println("Adding a threadgroup");
				engine.addThreadGroup((ThreadGroup) i.next());
			}
			println("Starting the test");
			engine.runTest();

		}catch(Exception e){
			System.out.println("Error in NonGUIDriver" + e.getMessage());
			e.printStackTrace();
		}
	}

	private static void println(String str){
		System.out.println(str);
	}

}