package org.apache.jmeter.control;

import java.util.*;
import org.apache.jmeter.samplers.*;
import org.apache.jmeter.util.ClassFinder;

public class DynamicController extends GenericController implements SampleListener
{
	private String currentResponse;
	private ResponseParser parser;

	/************************************************************
	 *  Constructor for the DynamicController object
	 ***********************************************************/
	public DynamicController()
	{
	}

	/************************************************************
	 *  Sets the Parser attribute of the DynamicController object
	 *
	 *@param  p  The new Parser value
	 ***********************************************************/
	public void setParser(ResponseParser p)
	{
		parser = p;
	}

	/************************************************************
	 *  Gets the ClassLabel attribute of the DynamicController object
	 *
	 *@return    The ClassLabel value
	 ***********************************************************/
	public String getClassLabel()
	{
		return "Spider Control";
	}

	/************************************************************
	 *  Gets the Parser attribute of the DynamicController object
	 *
	 *@return    The Parser value
	 ***********************************************************/
	public ResponseParser getParser()
	{
		return parser;
	}

	/************************************************************
	 *  Gets the GuiClass attribute of the DynamicController object
	 *
	 *@return    The GuiClass value
	 ***********************************************************/
	public Class getGuiClass()
	{
		return null;
		//return org.apache.jmeter.control.gui.DynamicControllerGui.class;
	}

	/************************************************************
	 *  Gets the ParserChoices attribute of the DynamicController object
	 *
	 *@return    The ParserChoices value
	 ***********************************************************/
	public List getParserChoices()
	{
		try
		{
			List choices = ClassFinder.findClassesThatExtend(
					new Class[]{org.apache.jmeter.control.ResponseParser.class});
			Iterator iter = choices.iterator();
			List choiceLabels = new LinkedList();
			while (iter.hasNext())
			{
				choiceLabels.add(((ResponseParser)Class.forName(
						(String)iter.next()).newInstance()).getClassLabel());
			}
			return choiceLabels;
		}
		catch (Exception ex)
		{
			return Collections.EMPTY_LIST;
		}
	}


	/************************************************************
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 ***********************************************************/
	public void sampleOccurred(SampleEvent e)
	{
		currentResponse = new String((byte[])e.getResult().getValue(SampleResult.TEXT_RESPONSE));
	}


	/************************************************************
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 ***********************************************************/
	public void sampleStarted(SampleEvent e)
	{
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 ***********************************************************/
	public void sampleStopped(SampleEvent e)
	{
	}


	/************************************************************
	 *  Adds a feature to the SelfOrNot attribute of the DynamicController object
	 *
	 *@param  listeners  The feature to be added to the SelfOrNot attribute
	 ***********************************************************/
	protected void addSelfOrNot(Collection listeners)
	{
		listeners.add(this);
	}

	/************************************************************
	 *  Adds a feature to the ConfigElements attribute of the DynamicController
	 *  object
	 *
	 *@param  entry  The feature to be added to the ConfigElements attribute
	 ***********************************************************/

	protected void addConfigElements(Entry entry)
	{
		super.addConfigElements(entry);
		try
		{
			parser.modifyEntry(entry, currentResponse, null);
		}
		catch (Exception e)
		{
		}
	}

}

