/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.control;

import java.util.*;
import java.io.*;

import org.apache.jmeter.gui.*;
import org.apache.jmeter.control.gui.*;

/************************************************************
 *  Title: JMeter Description: Copyright: Copyright (c) 2000 Company: Apache
 *
 *@author     Michael Stover
 *@created    March 13, 2001
 *@version    1.0
 ***********************************************************/

public class WorkBench implements JMeterComponentModel,Serializable
{
	private String name;
	private boolean isRootNode;
	private static List itemsCanAdd = null;

	/************************************************************
	 *  Constructor for the WorkBench object
	 *
	 *@param  name  Description of Parameter
	 ***********************************************************/
	public WorkBench(String name, boolean isRootNode)
	{
		this.name = name;
		this.isRootNode = isRootNode;
	}

	/************************************************************
	 *  Sets the Name attribute of the WorkBench object
	 *
	 *@param  name  The new Name value
	 ***********************************************************/
	public void setName(String name)
	{
		this.name = name;
	}

	/************************************************************
	 *  Sets the isRootNode attribute of the WorkBench object
	 *
	 *@param  name  The new isRootNode value
	 ***********************************************************/
	public void setIsRootElement(boolean isRootNode)
	{
		this.isRootNode = isRootNode;
	}

	/************************************************************
	 *  Gets the Name attribute of the WorkBench object
	 *
	 *@return    The Name value
	 ***********************************************************/
	public String getName()
	{
		return name;
	}

	/************************************************************
	 *  Gets the isRootNode attribute of the WorkBench object
	 *
	 *@return    The isRootNode value
	 ***********************************************************/
	public boolean isRootNode()
	{
		return this.isRootNode;
	}

	/************************************************************
	 *  Gets the ClassLabel attribute of the WorkBench object
	 *
	 *@return    The ClassLabel value
	 ***********************************************************/
	public String getClassLabel()
	{
		return "WorkBench";
	}

	/************************************************************
	 *  Indicates whether the item is editable. If so, the popup will include a
	 *  "edit" choice which will display the item's gui screen in the main panel.
	 *
	 *@return    The Editable value
	 ***********************************************************/
	public boolean isEditable()
	{
		return true;
	}

	/************************************************************
	 *  Gets the GuiClass attribute of the WorkBench object
	 *
	 *@return    The GuiClass value
	 ***********************************************************/
	public Class getGuiClass()
	{
		return org.apache.jmeter.control.gui.WorkBenchGui.class;
	}

	/************************************************************
	 *  Returns the list of object types that can be added to this item.
	 *
	 *@return    The AddList value
	 ***********************************************************/
	public Collection getAddList()
	{
		if(itemsCanAdd == null){
			itemsCanAdd = new LinkedList();
			itemsCanAdd.add(MenuFactory.getLogicControllerMenu());
			itemsCanAdd.add(MenuFactory.getGenerativeControllerMenu());
			itemsCanAdd.add(MenuFactory.getConfigElementMenu());
		}
		return itemsCanAdd;
	}

	public void addJMeterComponent(JMeterComponentModel child)
	{
	}

	/************************************************************
	 *  Description of the Method
	 ***********************************************************/
	public void uncompile()
	{
	}
}
