/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.ejb.jndi.save;

import java.io.IOException;
import java.io.Writer;
import java.util.List;

import org.apache.jmeter.control.GenericController;
import org.apache.jmeter.ejb.jndi.config.JndiConfig;
import org.apache.jmeter.ejb.jndi.control.JndiTestSample;
import org.apache.jmeter.save.handlers.GenericControllerHandler;
import org.apache.jmeter.save.handlers.JMeterHandler;
import org.apache.jmeter.save.xml.TagHandler;
import org.apache.jmeter.save.Saveable;
import org.apache.log4j.Category;
/**
 * Saves the JNDI sampling component for use later
 */
public class JndiTestSampleHandler extends GenericControllerHandler
{
  public JndiTestSampleHandler()
  {
  }

  public String getPrimaryTagName()
  {
    return "JndiTestSample";
  }

  /**
   * Called by <code>org.apache.jmeter.save.xml.XmlHandler</code> when
   * finish processing its current handler.
   *
   * @param childObj	
   */
  public void notifySubElementEnded(Object childObj)
  {
    System.out.println(childObj);
    if(this.getMethodName().endsWith("defaultJndi"))
    {
      retrieveDefaultJndiConfig();
    }
  }

  /**
   *
   */
  public void defaultJndiTagEnd()
  {
    retrieveDefaultJndiConfig();
  }

  /**
   * Saves the <code>Saveable</code> component
   * 
   * @param cont	component to be saved
   * @param out		<code>Writer</code> where config will be written out to
   */
  public void save(Saveable cont, Writer out) throws java.io.IOException
  {
    JndiTestSample controller = (JndiTestSample)cont;
    writeMainTag(out, controller);
    writeDefaultJndi(out, controller);
    writeSubElements(controller, out);
    out.write("</JndiTestSample>\n");
  }

  /**
   * The main tag of the controller to be written out
   * 
   * @param out 	<code>Writer</code> where config will be written out to
   * @param controller	controller to be written out
   */
  protected void writeMainTag(Writer out, GenericController controller) 
	throws java.io.IOException
  {
    out.write("<JndiTestSample type=\"");
    out.write(JMeterHandler.convertToXML(controller.getClass().getName()));
    out.write("\" name=\"");
    out.write(JMeterHandler.convertToXML(controller.getName()));
    out.write("\">\n");
  }

  /**
   * Saves the default Jndi config of JndiSampleTest controller
   *
   * @param out		<code>Writer</code> where config will be written out to
   * @param controller	controller to be written out
   */
  protected void writeDefaultJndi(Writer out, JndiTestSample controller)
	throws java.io.IOException
  {
    out.write("<defaultJndi>\n");
    JMeterHandler.writeObject(controller.getDefaultJndiConfig(), out);
    out.write("</defaultJndi>\n");
  }

  /**
   * Write the default <code>JndiConfig</code> out for saving as well
   */
  protected void retrieveDefaultJndiConfig()
  {
    List children = xmlParent.takeChildObjects(this);
    if(children.size() == 1)
    {
      ((JndiTestSample)controller).setDefaultJndiConfig((JndiConfig)
        ((TagHandler)children.get(0)).getModel());
    }
  }
}
