/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.engine;

import java.util.Iterator;
import java.util.List;

import org.apache.jmeter.config.ConfigElement;
import org.apache.jmeter.control.GenericController;
import org.apache.jmeter.control.LoopController;
import org.apache.jmeter.control.SamplerController;
import org.apache.jmeter.ejb.jndi.config.MethodConfig;
import org.apache.jmeter.ejb.jndi.control.JndiTestSample;
import org.apache.jmeter.threads.JMeterThread;
import org.apache.jmeter.threads.ThreadGroup;
import org.apache.log4j.Category;

/**
 * This engine looks for Remote Method <code>ConfigElement</code> and
 * runs reflection on it.  
 * (@see org.apache.jmeter.ejb.jndi.config.MethodConfigGui)
 *
 * @author		Khor Soon Hin
 * @created		2002/01/12
 * @version		1.0
 */
public class ReflectionJMeterEngine extends StandardJMeterEngine
{
  private static Category catClass = 
    Category.getInstance(ReflectionJMeterEngine.class.getName());


  /**
   * Constructor
   **/
  public ReflectionJMeterEngine()
  {
    super();
    catClass.debug("Start : ReflectionJMeterEngine1");
    catClass.debug("End : ReflectionJMeterEngine1");
  }

  /**
   * Run the engine
   **/
  public void runTest()
  {
    catClass.debug("Start : runTest1");
    // variables to store the ThreadGroups LoopController values so
    // that they can be reset later
    int lcLoops = 0;
    boolean lcContinueForever = false;
    running = true;
    JMeterThread jMeterThread;
    Iterator iter = threadGroups.iterator();
    ThreadGroup thisThreadGroup = null;
    boolean runThis = false;
    while(iter.hasNext())
    {
      // look for threadGroup with MethodConfig element
      thisThreadGroup = (ThreadGroup)iter.next();
      GenericController control = (GenericController)
		thisThreadGroup.getSamplerController().clone();
      List subControllerList = control.getSubControllers();
      List configElementList = control.getConfigElements();
      Iterator subControllersIterator = subControllerList.iterator();
      if(catClass.isDebugEnabled())
      { 
        catClass.debug("runTest1 : thisThreadGroup - " + thisThreadGroup);
        catClass.debug("runTest1 : controller - " + control);
        catClass.debug("runTest1 : configElementList - " + configElementList);
        catClass.debug("runTest1 : subControllerList - " + subControllerList);
      }
      while(subControllersIterator.hasNext())
      {
        SamplerController samplerController = (SamplerController)
		subControllersIterator.next();
        if(catClass.isDebugEnabled())
        {
          catClass.debug("runTest1 : samplerController type - " 
		+ samplerController.getClass());
        }
        if(samplerController instanceof JndiTestSample)
        {
          runThis = true;
        }
      }
      if(runThis)
      {
        catClass.debug("runTest1 : running ThreadGroup with JndiTestSample");
        // Change the ThreadGroup looping to 1 'cos you only need to
        // run the JNDISampler once to do reflection
        // Ensure that sampleController of ThreadGroup is a LoopController
        // which is a certainty but still in case things change in the future
        // this should flag it
        LoopController lc = null;
        if(control instanceof LoopController)
        {
          lc = (LoopController)control;
          lcLoops = lc.getLoops();
          lcContinueForever = lc.getContinueForever();
          lc.setContinueForever(false);
	  lc.setLoops(1);
          thisThreadGroup.setSamplerController(lc);
        }
        runThis = false;
        jMeterThread = new JMeterThread(thisThreadGroup);
        Thread newThread = new Thread(jMeterThread);
        newThread.start();
        try
        {
          newThread.join();
        }
        catch(InterruptedException e)
        {
          catClass.error(e);
          System.out.println(e);
        }
        lc.setContinueForever(lcContinueForever);
        lc.setLoops(lcLoops);
        thisThreadGroup.setSamplerController(lc);
      }
    }
    catClass.debug("End : runTest1");
  }
}
