/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.gui;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

import org.apache.jmeter.util.JMeterUtils;
import org.apache.jmeter.control.LoopController;

/************************************************************
 *  Title: JMeter Description: Copyright: Copyright (c) 2000 Company: Apache
 *
 *@author     Michael Stover
 *@created    $Date: 2001/09/27 13:16:34 $
 *@version    1.0
 ***********************************************************/

public class LoopControlPanel extends JPanel implements ModelSupported, KeyListener, ActionListener
{
	LoopController model;
	JCheckBox infinite;
	JTextField loops;

	private boolean displayName = true;
	private static String INFINITE = "Infinite Field";
	private static String LOOPS = "Loops Field";

	/************************************************************
	 *  !ToDo (Constructor description)
	 ***********************************************************/
	public LoopControlPanel()
	{
	}

	/************************************************************
	 *  !ToDo (Constructor description)
	 *
	 *@param  displayName  !ToDo (Parameter description)
	 ***********************************************************/
	public LoopControlPanel(boolean displayName)
	{
		this.displayName = displayName;
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  model  !ToDo (Parameter description)
	 ***********************************************************/
	public void setModel(Object model)
	{
		this.model = (LoopController)model;
		init();
	}

	/************************************************************
	 *  !ToDo (Method description)
	 ***********************************************************/
	public void updateGui()
	{
		setState();
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  event  !ToDo (Parameter description)
	 ***********************************************************/
	public void actionPerformed(ActionEvent event)
	{
		if (infinite.isSelected())
		{
			loops.setText("");
			model.setLoops(-1);
			loops.setEnabled(false);
		}
		else
		{
			loops.setEnabled(true);
			new FocusRequester(loops);
		}
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 ***********************************************************/
	public void keyPressed(KeyEvent e)
	{
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 ***********************************************************/
	public void keyTyped(KeyEvent e)
	{
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 ***********************************************************/
	public void keyReleased(KeyEvent e)
	{
		String temp = e.getComponent().getName();
		if (temp.equals(LOOPS))
		{
			try
			{
				model.setLoops(Integer.parseInt(loops.getText()));
			}
			catch (NumberFormatException ex)
			{
				if (loops.getText().length() > 0)
				{
					// We need a standard warning/error dialog. The problem with
					// having it here is that the dialog is centered over this
					// LoopControlPanel instead of begin centered in the entire
					// JMeter GUI window.
					JOptionPane.showMessageDialog(this, "You must enter a valid number",
						"Invalid data", JOptionPane.WARNING_MESSAGE);
					model.setLoops(-1);
					loops.setText("");
				}
				else
				{
					model.setLoops(-1);
				}
			}
		}
	}

	private void init()
	{
		// The Loop Controller panel can be displayed standalone or inside another panel.
		// For standalone, we want to display the TITLE, NAME, etc. (everything). However,
		// if we want to display it within another panel, we just display the Loop Count
		// fields (not the TITLE and NAME).

		// Standalone
		if (displayName)
		{
			this.setLayout(new VerticalLayout(5, VerticalLayout.LEFT, VerticalLayout.TOP));

			// MAIN PANEL
			JPanel mainPanel = new JPanel();
			Border margin = new EmptyBorder(10, 10, 5, 10);
			mainPanel.setBorder(margin);
			mainPanel.setLayout(new VerticalLayout(5, VerticalLayout.LEFT));

			// TITLE
			JLabel panelTitleLabel = new JLabel(JMeterUtils.getResString("loop_controller_title"));
			Font curFont = panelTitleLabel.getFont();
			int curFontSize = curFont.getSize();
			curFontSize += 4;
			panelTitleLabel.setFont(new Font(curFont.getFontName(), curFont.getStyle(), curFontSize));
			mainPanel.add(panelTitleLabel);

			// NAME
			NamePanel namePanel = new NamePanel(model);
			mainPanel.add(namePanel);

			// LOOP
			mainPanel.add(createLoopCountPanel());

			this.add(mainPanel);
		}
		
		// Embedded
		else
		{
			this.add(createLoopCountPanel());
		}
	}

	private JPanel createLoopCountPanel()
	{
		JPanel loopPanel = new JPanel();

		// LOOP LABEL
		JLabel loopsLabel = new JLabel(JMeterUtils.getResString("iterator_num"));
		loopPanel.add(loopsLabel);
		
		// TEXT FIELD
		loops = new JTextField(5);
		loopPanel.add(loops);
		loops.setName(LOOPS);
		loops.addKeyListener(this);

		// FOREVER CHECKBOX
		infinite = new JCheckBox(JMeterUtils.getResString("infinite"));
		infinite.setActionCommand(INFINITE);
		infinite.addActionListener(this);
		loopPanel.add(infinite);

		return loopPanel;
	}

	private void setState()
	{
		if (model.getLoops() <= -1)
		{
			infinite.setSelected(true);
			loops.setEnabled(false);
			loops.setText("");
		}
		else
		{
			infinite.setSelected(false);
			loops.setEnabled(true);
			loops.setText(""+model.getLoops());
		}
	}
}
