/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
 package org.apache.jmeter.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;

import org.apache.jmeter.gui.tree.*;
import org.apache.jmeter.gui.action.ActionRouter;
/************************************************************
 *  Title: JMeter Description: Copyright: Copyright (c) 2000 Company: Apache
 *
 *@author     Michael Stover
 *@created    $Date: 2002/03/02 19:33:54 $
 *@version    1.0
 ***********************************************************/

public class MainFrame extends JFrame
{
	JPanel all, mainPanel, toolPanel;
	JScrollPane treePanel;
	JMeterMenuBar menuBar;
	JTree tree;
	TreeModel treeModel;
	ActionListener actionHandler;
	JMeterTreeListener treeListener;
	private boolean running;

	/************************************************************
	 *  !ToDo (Constructor description)
	 *
	 *@param  menuModel  !ToDo (Parameter description)
	 ***********************************************************/
	public MainFrame(ActionListener actionHandler,TreeModel treeModel,JMeterTreeListener
			treeListener)
	{
		this.treeListener = treeListener;
		this.actionHandler = actionHandler;
		this.treeModel = treeModel;
		init();
	}

	public void setFileSaveEnabled(boolean enabled)
	{
		menuBar.setFileSaveEnabled(enabled);
	}

	public void setFileLoadEnabled(boolean enabled)
	{
		menuBar.setFileLoadEnabled(enabled);
	}

	public void setEditAddMenu(JMenu menu)
	{
		menuBar.setEditAddMenu(menu);
	}

	public void setEditAddEnabled(boolean enabled)
	{
		menuBar.setEditAddEnabled(enabled);
	}

	public void setEditRemoveEnabled(boolean enabled)
	{
		menuBar.setEditRemoveEnabled(enabled);
	}

	public void setEditEnabled(boolean enabled)
	{
		menuBar.setEditEnabled(enabled);
	}

	public void closeMenu()
	{
		if (menuBar.isSelected())
		{
			MenuElement[] menuElement = menuBar.getSubElements();
			if (menuElement != null)
			{
				for (int i = 0; i < menuElement.length; i++)
				{
					JMenu menu = (JMenu)menuElement[i];
					if (menu.isSelected())
					{
						menu.setPopupMenuVisible(false);
						menu.setSelected(false);
						break;
					}
				}
			}
		}
	}

	public void setMainPanel(JComponent comp)
	{
		mainPanel.removeAll();
		Rectangle rect = mainPanel.getVisibleRect();
		Insets in = mainPanel.getInsets();
		Dimension dim = new Dimension(((int)rect.getWidth())-(in.right+in.left)*2,
						  ((int)rect.getHeight())-(in.top+in.bottom)*2);
		Dimension min = comp.getMinimumSize();
		if(dim.width < min.width)
		{
			dim.width = min.width;
		}
		if(dim.height < min.height)
		{
			dim.height = min.height;
		}
		comp.setPreferredSize(dim);
		comp.validate();
		mainPanel.add(comp);
		all.validate();
		all.repaint();
	}

	private void init()
	{
		menuBar = new JMeterMenuBar();
		createToolBar();
		createTreePanel();
		createMainPanel();
		addThemAll();
		addWindowListener(new WindowHappenings());
	}

	private TreeCellRenderer getCellRenderer()
	{
		DefaultTreeCellRenderer rend = new JMeterCellRenderer();
		rend.setFont(new Font("Dialog",Font.PLAIN,11));
		return rend;
	}

	private void addThemAll()
	{
		this.setJMenuBar(menuBar);
		all = new JPanel(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		{
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.gridheight = 1;
			gbc.gridwidth = 1;
			gbc.fill = gbc.BOTH;
			gbc.weightx = 1;
			gbc.weighty = 0;
			all.add(toolPanel,gbc.clone());
		}
		JSplitPane treeAndMain = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		treeAndMain.setLeftComponent(treePanel);
		treeAndMain.setRightComponent(new JScrollPane(mainPanel));

		// The setResizeWeight() method was added to JDK1.3. For now, JMeter should
		// remain compatible with JDK1.2.
		//treeAndMain.setResizeWeight(.2);

		treeAndMain.setContinuousLayout(true);
		{
			gbc.gridy++;
			gbc.weighty = 1;
			all.add(treeAndMain,gbc.clone());
		}
		this.getContentPane().add(all);
	}

	private void createToolBar()
	{
		toolPanel = new JPanel();
		toolPanel.add(Box.createHorizontalGlue());
	}

	private void createTreePanel()
	{
		treePanel = new JScrollPane(makeTree());
	}

	private void createMainPanel()
	{
		mainPanel = new JPanel();
		mainPanel.setLayout(new GridLayout(1,1));
		mainPanel.add(Box.createGlue());
	}

	private JTree makeTree()
	{
		tree = new JTree(this.treeModel);
		tree.setCellRenderer(getCellRenderer());
		treeListener.setJTree(tree);
		tree.addTreeSelectionListener(treeListener);
		tree.addMouseListener(treeListener);
		tree.addKeyListener(treeListener);
		return tree;
	}

	public JTree getTree()
	{
		return tree;
	}

	public void setRunning(boolean newRunning,String host)
	{
		this.menuBar.setRunning(newRunning,host);
	}

	public void setRunning(boolean newRunning)
	{
		running = newRunning;
		this.menuBar.setEnabled(newRunning);
	}

	private class WindowHappenings extends WindowAdapter
	{
		public void windowClosing(WindowEvent event)
		{
			ActionRouter.getInstance().actionPerformed(new ActionEvent(
			this,event.getID(),"exit"));
		}
	}
}
