/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.gui.tree;

import java.util.*;

import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;

import org.apache.jmeter.gui.*;
import org.apache.jmeter.config.ConfigElement;
import org.apache.jmeter.control.WorkBench;
import org.apache.jmeter.control.TestPlan;
import org.apache.jmeter.save.Saveable;
import org.apache.jmeter.timers.*;
import org.apache.jmeter.util.JMeterUtils;

/**
 *  Title: JMeter Description: Copyright: Copyright (c) 2000 Company: Apache
 *
 *@author     Michael Stover
 *@created    March 11, 2001
 *@version    1.0
 */

public class JMeterTreeListener implements TreeSelectionListener, MouseListener, KeyListener
{
	Container endWindow;
	JPopupMenu pop;
	TreePath currentPath;
	ActionListener actionHandler;

	private JMeterTreeModel model;
	private JTree tree;

	/**
	 *  Constructor for the JMeterTreeListener object
	 *
	 *@param  model  Description of Parameter
	 */
	public JMeterTreeListener(JMeterTreeModel model)
	{
		this.model = model;
	}


	/**
	 *  Sets the ActionHandler attribute of the JMeterTreeListener object
	 *
	 *@param  ah  The new ActionHandler value
	 */
	public void setActionHandler(ActionListener ah)
	{
		actionHandler = ah;
	}


	/**
	 *  Sets the JTree attribute of the JMeterTreeListener object
	 *
	 *@param  tree  The new JTree value
	 */
	public void setJTree(JTree tree)
	{
		this.tree = tree;
	}


	/**
	 *  Sets the EndWindow attribute of the JMeterTreeListener object
	 *
	 *@param  window  The new EndWindow value
	 */
	public void setEndWindow(Container window)
	{
		endWindow = window;
	}


	/**
	 *  Gets the CurrentNode attribute of the JMeterTreeListener object
	 *
	 *@return    The CurrentNode value
	 */
	public JMeterTreeNode getCurrentNode()
	{
		if (currentPath != null)
		{
			if(currentPath.getLastPathComponent() != null)
			{
				return (JMeterTreeNode) currentPath.getLastPathComponent();
			}
			else
			{
				return (JMeterTreeNode)currentPath.getParentPath().getLastPathComponent();
			}
		}
		else
		{
			return (JMeterTreeNode) model.getRoot();
		}
	}

	public JMeterTreeNode[] getSelectedNodes()
	{
		TreePath[] paths = tree.getSelectionPaths();
		if(paths == null)
		{
			return new JMeterTreeNode[]{getCurrentNode()};
		}
		JMeterTreeNode[] nodes = new JMeterTreeNode[paths.length];
		for (int i = 0; i < paths.length; i++)
		{
			nodes[i] = (JMeterTreeNode)paths[i].getLastPathComponent();
		}
		return nodes;
	}

	public void removedSelectedNode()
	{
		currentPath = currentPath.getParentPath();
	}


	/**
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 */
	public void valueChanged(TreeSelectionEvent e)
	{
		currentPath = e.getNewLeadSelectionPath();
		actionHandler.actionPerformed(new ActionEvent(this, 3333, JMeterUtils.getResString("edit")));
	}


	/**
	 *  Description of the Method
	 *
	 *@param  ev  Description of Parameter
	 */
	public void mouseClicked(MouseEvent ev)
	{
	}


	/**
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 */
	public void mouseReleased(MouseEvent e)
	{
	}


	/**
	 *  Description of the Method
	 *
	 *@param  ev  Description of Parameter
	 */
	public void mouseEntered(MouseEvent ev)
	{
	}


	/**
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 */
	public void mousePressed(MouseEvent e)
	{
		// Get the Main Frame.
		MainFrame mainFrame = GuiPackage.getInstance().getMainFrame();

		// Close any Main Menu that is open
		mainFrame.closeMenu();
		int selRow = tree.getRowForLocation(e.getX(), e.getY());
		if (tree.getPathForLocation(e.getX(), e.getY()) != null)
		{
			currentPath = tree.getPathForLocation(e.getX(), e.getY());
		}
		actionHandler.actionPerformed(new ActionEvent(this, e.getID(), JMeterUtils.getResString("edit")));
		if (selRow != -1)
		{
			if (isRightClick(e))
			{
				tree.setSelectionPath(currentPath);
				updateMainMenu(createAddMenu((JMeterComponentModel) (getCurrentNode().getUserObject())));
				rightClick(currentPath, createAddMenu((JMeterComponentModel) (getCurrentNode().getUserObject())), e.getX(), e.getY());
			}
			else
			{
				updateMainMenu(createAddMenu((JMeterComponentModel) (getCurrentNode().getUserObject())));
			}
		}
	}


	/**
	 *  Description of the Method
	 *
	 *@param  ev  Description of Parameter
	 */
	public void mouseExited(MouseEvent ev)
	{
	}

	/**
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 */
	public void keyPressed(KeyEvent e)
	{
	}

	/**
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 */
	public void keyReleased(KeyEvent e)
	{
	}

	/**
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 */
	public void keyTyped(KeyEvent e)
	{
	}


	/**
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 *@return    Description of the Returned Value
	 */
	private boolean isRightClick(MouseEvent e)
	{
		return (MouseEvent.BUTTON2_MASK & e.getModifiers()) > 0 ||
				(MouseEvent.BUTTON3_MASK == e.getModifiers());
	}


	/**
	 *  Description of the Method
	 *
	 *@param  addMenu  Description of Parameter
	 */
	private void updateMainMenu(JMenu addMenu)
	{
		try
		{
			Object model = getCurrentNode().getUserObject();
			MainFrame mainFrame = GuiPackage.getInstance().getMainFrame();

			// Disable/Enable the FILE-->SAVE menu item
			if (!(model instanceof Saveable))
			{
				mainFrame.setFileSaveEnabled(false);
			}
			else
			{
				mainFrame.setFileSaveEnabled(true);
			}

			// Disable/Enable the FILE-->OPEN menu item
			if ((model instanceof WorkBench))
			{
				// Disable the Open menu item if the node is Root.
				// Otherwise, enable the menu item because it is the WorkBench node.
				if (((WorkBench) model).isRootNode())
				{
					mainFrame.setFileLoadEnabled(false);
				}
				else
				{
					mainFrame.setFileLoadEnabled(true);
				}
			}
			else
			{
				// Enable the Open menu item if the node has ADD items (meaning it is not
				// a leaf node)
				JMeterComponentModel tempModel = (JMeterComponentModel) model;
				if ((tempModel.getAddList() != null) && (tempModel.getAddList().size() > 0))
				{
					mainFrame.setFileLoadEnabled(true);
				}
				else
				{
					mainFrame.setFileLoadEnabled(false);
				}
			}

			// Disable/Enable the EDIT-->ADD and EDIT-->REMOVE menu items
			JMeterComponentModel tempModel = (JMeterComponentModel) model;
			if ((model instanceof WorkBench))
			{
				// Disable the Edit Menu, and  Edit Add and Edit Remove menu items
				//  if the node is Root. Otherwise, enable the Add and Remove menu
				// items because the node is the WorkBench node.
				if (((WorkBench) model).isRootNode())
				{
					mainFrame.setEditEnabled(false);
					mainFrame.setEditAddEnabled(false);
					mainFrame.setEditRemoveEnabled(false);
				}
				else
				{
					// If the Edit-->Add menu does not exist the create it.
					//mainFrame.setEditAddMenu(getAddMenu(tempModel));
					mainFrame.setEditAddMenu(addMenu);
					mainFrame.setEditAddEnabled(true);
					mainFrame.setEditRemoveEnabled(false);
				}
			}
			else
			{
				// Enable the Add menu item if the node has ADD items (meaning it is
				// not a leaf node).
				if ((tempModel.getAddList() != null) && (tempModel.getAddList().size() > 0))
				{
					mainFrame.setEditAddMenu(addMenu);
					mainFrame.setEditAddEnabled(true);
				}
				else
				{
					mainFrame.setEditAddEnabled(false);
				}

				// Disable the Remove menu item if the node is TestPlan. Otherwise, enable it.
				if (tempModel instanceof TestPlan)
				{
					mainFrame.setEditRemoveEnabled(false);
				}
				else
				{
					mainFrame.setEditRemoveEnabled(true);
				}
			}
		}
		catch (NullPointerException e)
		{
			System.out.println("Null pointer: JMeterTreeListener.updateMenuItem()");
			e.printStackTrace();
		}
	}


	/**
	 *  Description of the Method
	 *
	 *@param  selPath  Description of Parameter
	 *@param  x        Description of Parameter
	 *@param  y        Description of Parameter
	 *@param  addMenu  Description of Parameter
	 */
	private void rightClick(TreePath selPath, JMenu addMenu, int x, int y)
	{
		Object node = ((JMeterTreeNode) selPath.getLastPathComponent()).getUserObject();
		if (node instanceof JMeterComponentModel)
		{
			boolean showPopup = true;

			// Do not show the popup if the user right-clicks on the root node
			if (node instanceof WorkBench)
			{
				boolean isRootNode = ((WorkBench) node).isRootNode();
				showPopup = !isRootNode;
			}

			if (showPopup)
			{
				generatePopupMenu((JMeterComponentModel) node, addMenu, x, y);
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  comp     Description of Parameter
	 *@param  x        Description of Parameter
	 *@param  y        Description of Parameter
	 *@param  addMenu  Description of Parameter
	 */
	private void generatePopupMenu(JMeterComponentModel comp, JMenu addMenu, int x, int y)
	{
		pop = new JPopupMenu();
		pop.setInvoker(tree);

		// Get the OPEN menu item.
		JMenuItem loadMenuItem = null;
		boolean showLoadMenuItem = false;
		if ((comp instanceof WorkBench))
		{
			if (((WorkBench) comp).isRootNode())
			{
				showLoadMenuItem = false;
			}
			else
			{
				showLoadMenuItem = true;
			}
		}
		else
		{
			// If the node has ADD items (meaning it is not a leaf node), then
			// show the menu item.
			if ((comp.getAddList() != null) && (comp.getAddList().size() > 0))
			{
				showLoadMenuItem = true;
			}
			else
			{
				showLoadMenuItem = false;
			}
		}

		if (showLoadMenuItem)
		{
			loadMenuItem = new JMenuItem(JMeterUtils.getResString("open"));
			loadMenuItem.addActionListener(actionHandler);
		}

		// Get the SAVE menu item.
		JMenuItem saveMenuItem = null;
		if (comp instanceof Saveable)
		{
			saveMenuItem = new JMenuItem(JMeterUtils.getResString("save"));
			saveMenuItem.addActionListener(actionHandler);
		}

		// Get REMOVE menu item.
		JMenuItem removeMenuItem = null;
		if ((!(comp instanceof WorkBench)) && (!(comp instanceof TestPlan)))
		{
			removeMenuItem = new JMenuItem(JMeterUtils.getResString("remove"));
			removeMenuItem.addActionListener(actionHandler);
		}

		// Add the menu items.
		if (addMenu != null)
		{
			pop.add(addMenu);
		}

		if (loadMenuItem != null)
		{
			// If ADD menu is displayed, put a separator before the OPEN menu item
			if (addMenu != null)
			{
				pop.addSeparator();
			}

			pop.add(loadMenuItem);
		}

		if (saveMenuItem != null)
		{
			// If ADD menu is displayed and the OPEN menu item is not displayed,
			// put a separator before the SAVE menu item
			if ((addMenu != null) && (loadMenuItem == null))
			{
				pop.addSeparator();
			}

			pop.add(saveMenuItem);
		}

		if (removeMenuItem != null)
		{
			// If any menu item is displayed, then we need to put a put a separator
			// before the REMOVE menu item
			if ((addMenu != null) || (loadMenuItem != null) || (saveMenuItem != null))
			{
				pop.addSeparator();
			}

			pop.add(removeMenuItem);
		}

		pop.validate();
		pop.setPopupSize(pop.getPreferredSize());
		pop.show(tree, x, y);
		pop.setVisible(true);
		pop.requestFocus();
	}


	/**
	 *  Description of the Method
	 */
	private void releasePopup()
	{
		pop.setVisible(false);
		tree.requestFocus();
		pop.validate();
	}

	/**
	 *  Gets the AddMenu attribute of the JMeterTreeListener object
	 *
	 *@param  comp  Description of Parameter
	 *@return       Description of the Returned Value
	 */
	private JMenu createAddMenu(JMeterComponentModel comp)
	{
		JMenu menu = null;

		// Create an ADD menu if the component has ADD items.
		if (comp.getAddList() != null && comp.getAddList().size() > 0)
		{

			// Create menu
			menu = new JMenu(JMeterUtils.getResString("add"));

			// Create menu item for each ADD item.
			for (Iterator i = comp.getAddList().iterator(); i.hasNext(); )
			{
				Object compToAdd = i.next();

				// Menu item
				if (compToAdd instanceof String)
				{
					JMenuItem item = new JMenuItem((String) compToAdd);
					item.addActionListener(actionHandler);
					menu.add(item);
				}

				// Sub menu
				else if (compToAdd instanceof JMenu)
				{

					// The following line is causing the problem when you right-click
					// a tree element, immediately followed by a click on the
					// Edit--> Add menu. The Add menu contains no items!
					menu.add((JMenu) compToAdd);
				}
			}
		}

		return menu;
	}
}
