/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
 package org.apache.jmeter.protocol.ftp.control;

import java.util.*;
import java.io.*;

import org.apache.jmeter.control.AbstractGenerativeController;
import org.apache.jmeter.gui.MenuFactory;
import org.apache.jmeter.config.*;
import org.apache.jmeter.protocol.ftp.config.*;
import org.apache.jmeter.protocol.ftp.control.gui.*;
import org.apache.jmeter.samplers.*;
import org.apache.jmeter.protocol.ftp.sampler.FTPSampler;
import org.apache.jmeter.util.JMeterUtils;

/**
 * Title:        Apache JMeter
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Apache Foundation
 * @author Michael Stover
 * @version 1.0
 */

public class FtpTestSampler extends AbstractGenerativeController implements Serializable
{
	private List urls;
	private FtpConfig defaultUrl;
	private LoginConfig loginConfig;

	private static List addableList = new LinkedList();
	static
	{
		addableList.add(MenuFactory.getAssertionMenu());
	}

	public FtpTestSampler()
	{
		super();
		defaultUrl = new FtpConfig();
		loginConfig = new LoginConfig();
	}

	public Class getGuiClass()
	{
		return org.apache.jmeter.protocol.ftp.control.gui.FtpTestSamplerGui.class;
	}

	public void uncompile()
	{
		super.uncompile();
	}

	public void setDefaultUrl(FtpConfig config)
	{
		defaultUrl = config;
	}

	public FtpConfig getDefaultUrl()
	{
		return defaultUrl;
	}

	public void setLoginConfig(LoginConfig config)
	{
		loginConfig = config;
	}

	public LoginConfig getLoginConfig()
	{
		return loginConfig;
	}

	/**
	 *  Gets the ClassLabel attribute of the HttpTestSample object
	 *
	 *@return    The ClassLabel value
	 */
	public String getClassLabel() {
		return JMeterUtils.getResString("ftp_testing_title");
	}

	/**
	 *  Gets the AddList attribute of the HttpTestSample object
	 *
	 *@return    The AddList value
	 */
	public Collection getAddList() {
		return addableList;
	}

	public Class getTagHandlerClass()
	{
		return org.apache.jmeter.protocol.ftp.save.FtpTestHandler.class;
	}

	/**
	 *  !ToDo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 */
	public Object clone() {
		FtpTestSampler control = new FtpTestSampler();
		control.setDefaultUrl(defaultUrl);
		control.setLoginConfig(loginConfig);
		this.standardCloneProc(control);
		return control;
	}

		/**
	 *  Description of the Method
	 *
	 *@param  next  Description of Parameter
	 *@return       Description of the Returned Value
	 */
	protected Entry createEntry() {
		Entry entry = new Entry();
		entry.setSamplerClass(FTPSampler.class);
		entry.addConfigElement(defaultUrl);
		entry.addConfigElement(loginConfig);
		return entry;
	}

}
