/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.protocol.http.config;

import java.util.*;
import java.net.*;

import org.apache.jmeter.config.*;
import org.apache.jmeter.protocol.http.config.*;
import org.apache.jmeter.util.JMeterUtils;

import org.apache.log4j.*;
import java.io.*;

import junit.framework.TestCase;
import junit.framework.TestSuite;
/**
 *  Title: Apache JMeter Description: Copyright: Copyright (c) 2000 Company:
 *  Apache Foundation
 *
 *@author     Michael Stover
 *@created    $Date: 2002/02/23 01:21:06 $
 *@version    $Revision: 1.7 $
 */

public class UrlConfigFull extends UrlConfig implements Serializable
{
		  public static Category catClass =
			 Category.getInstance(UrlConfigFull.class.getName());

	/**
	 *  Constructor for the UrlConfigFull object
	 */
	public UrlConfigFull()
	{
			 super();
	}

		  /**
			* Return label
			*
			* @return label
			*/
		  public String getClassLabel()
		  {
		         return JMeterUtils.getResString("url_full_config_title");
		  }

	/**
	 *  Clone the instance
	 *
	 *@return    cloned instance
	 */
	public Object clone()
	{
					 catClass.debug("Start : clone1");
		UrlConfigFull newConfig = new UrlConfigFull();
		configureClone(newConfig);
					 catClass.debug("End : clone1");
		return newConfig;
	}

	/**
	 *  This allows config elements to combine and give a "layered" effect. for
	 *  example, say there are two HTTPConfigElements, which have properties for
	 *  domain, path, method, and parameters. If element A has everything filled
	 *  in, but null for domain, and element B is added, which has only domain
	 *  filled in, then after adding B to A, A will have the domain from B. If A
	 *  already had a domain, then the correct behavior is for A to ignore the
	 *  addition of element B.
	 *
	 * @param  config  config element to be added to
	 */
	public void addConfigElement(ConfigElement config)
	{
					 catClass.debug("Start : addConfigElement1");
					 if(catClass.isDebugEnabled())
					 {
						catClass.debug("addConfigElement1 : ConfigElement class - " + config.getClass());
					 }
		if (config instanceof UrlConfigFull)
		{
								catClass.debug("addConfigElement1 : ConfigElement is an instance of UrlConfigFull");
			updatePropertyIfAbsent((UrlConfigFull) config);
		}
		if (config instanceof UrlConfig)
		{
								catClass.debug("addConfigElement1 : ConfigElement is an instance of UrlConfig");
			updatePropertyIfAbsent((UrlConfig) config);
		}
					 catClass.debug("End : addConfigElement1");
	}

  public static class Test extends TestCase
  {
	 private UrlConfigFull urlConfigFull;
	 private UrlConfigFull urlConfigFullHalf;

	 private static Category catClass =
		Category.getInstance(Test.class.getName());

	 public Test(String name)
	 {
		super(name);
	 }

	 protected void setUp()
	 {
		catClass.debug("setUp1 : Start");
		urlConfigFull = new UrlConfigFull();
		urlConfigFull.setMethod(UrlConfig.GET);
		urlConfigFull.setProtocol("http");
		urlConfigFull.setPort(8080);
		urlConfigFull.setDomain("jakarta.apache.org");
		urlConfigFull.setPath("/index.html");

		urlConfigFullHalf = new UrlConfigFull();
		urlConfigFullHalf.setPath("/docs/index.html");
		catClass.debug("setUp1 : End");
	 }

	 public void testGetClassLabel()
	 {
		catClass.debug("Start : testGetClassLabel1");
		assertEquals("UrlFull Sample", urlConfigFull.getClassLabel());
		catClass.debug("End : testGetClassLabel1");
	 }

	 public void testClone()
	 {
		catClass.debug("Start : testClone1");
		UrlConfigFull newUrlConfigFull = (UrlConfigFull)urlConfigFull.clone();
		assertEquals(UrlConfig.GET, newUrlConfigFull.getMethod());
		assertEquals("http", newUrlConfigFull.getProtocol());
		assertEquals(8080, newUrlConfigFull.getPort());
		assertEquals("jakarta.apache.org", newUrlConfigFull.getDomain());
		assertEquals("/index.html", newUrlConfigFull.getPath());
		catClass.debug("End : testClone1");
	 }

	 public void testAddConfigElementWithUrlConfig()
	 {
		catClass.debug("Start : testAddConfigElementWithUrlConfig");
		UrlConfig urlConfig = new UrlConfig();
		urlConfig.setMethod(UrlConfig.GET);
		urlConfig.setProtocol("http");
		urlConfig.setPort(8080);
		urlConfig.setDomain("jakarta.apache.org");
		urlConfig.setPath("/index.html");
		urlConfigFullHalf.addConfigElement(urlConfig);
		assertEquals(UrlConfig.GET, urlConfigFullHalf.getMethod());
		assertEquals("http", urlConfigFullHalf.getProtocol());
		assertEquals(8080, urlConfigFullHalf.getPort());
		assertEquals("jakarta.apache.org", urlConfigFullHalf.getDomain());
		assertEquals("/docs/index.html", urlConfigFullHalf.getPath());
		catClass.debug("End : testAddConfigElementWithUrlConfig");
	 }

	 public void testAddConfigElementWithUrlConfigFull()
	 {
		urlConfigFullHalf.addConfigElement(urlConfigFull);
		assertEquals(UrlConfig.GET, urlConfigFullHalf.getMethod());
		assertEquals("http", urlConfigFullHalf.getProtocol());
		assertEquals(8080, urlConfigFullHalf.getPort());
		assertEquals("jakarta.apache.org", urlConfigFullHalf.getDomain());
		assertEquals("/docs/index.html", urlConfigFullHalf.getPath());
	 }
	 protected void tearDown()
	 {
		catClass.debug("Start : tearDown1");
		urlConfigFull = null;
		catClass.debug("End : tearDown1");
	 }

	 public static void main(String[] args)
	 {
		BasicConfigurator.configure();
		TestSuite suite = new TestSuite(Test.class);
		junit.textui.TestRunner runner = new junit.textui.TestRunner();
		runner.run(suite);
	 }
  }
}


