/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
 package org.apache.jmeter.protocol.jdbc.control;

import org.apache.jmeter.control.GenericController;
import org.apache.jmeter.protocol.jdbc.config.*;
import org.apache.jmeter.protocol.jdbc.control.gui.*;
import org.apache.jmeter.config.ConfigElement;
import org.apache.jmeter.samplers.Entry;
import org.apache.jmeter.samplers.CycleEntry;
import org.apache.jmeter.protocol.jdbc.sampler.JDBCSampler;
import org.apache.jmeter.gui.MenuFactory;
import org.apache.jmeter.control.AbstractGenerativeController;
import org.apache.jmeter.util.JMeterUtils;
import java.util.*;
import java.io.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Michael Stover
 * @version 1.0
 */

public class JdbcTestSample extends AbstractGenerativeController implements Serializable
{

	private DbConfig defaultDbConfig;
	private PoolConfig defaultPoolConfig;
	private SqlConfig defaultSqlConfig;

	private static Set addableList = new HashSet();
	static
	{
		addableList.add(MenuFactory.getAssertionMenu());
	}

	public JdbcTestSample()
	{
		defaultSqlConfig = new SqlConfig();
		defaultPoolConfig = new PoolConfig();
		defaultDbConfig = new DbConfig();
	}

	public SqlConfig getDefaultSqlConfig()
	{
		return defaultSqlConfig;
	}

	public void setDefaultSqlConfig(SqlConfig config)
	{
		defaultSqlConfig = config;
	}

	public PoolConfig getDefaultPoolConfig()
	{
		return defaultPoolConfig;
	}

	public DbConfig getDefaultDbConfig()
	{
		return defaultDbConfig;
	}

	public void setDefaultPoolConfig(PoolConfig config)
	{
		this.defaultPoolConfig = config;
	}

	public void setDefaultDbConfig(DbConfig config)
	{
		this.defaultDbConfig = config;
	}

	public void uncompile()
	{
		super.uncompile();
	}

	public String getClassLabel()
	{
		return JMeterUtils.getResString("database_testing_title");
	}

	public Class getGuiClass()
	{
		return org.apache.jmeter.protocol.jdbc.control.gui.JdbcTestSampleGui.class;
	}

	public Class getTagHandlerClass()
	{
		return org.apache.jmeter.protocol.jdbc.save.JdbcTestSampleHandler.class;
	}

	public Collection getAddList()
	{
		return addableList;
	}

	public Object clone() {
		JdbcTestSample control = new JdbcTestSample();
		control.setDefaultPoolConfig(defaultPoolConfig);
		control.setDefaultDbConfig(defaultDbConfig);
		control.setDefaultSqlConfig(defaultSqlConfig);
		this.standardCloneProc(control);
		return control;
	}

	protected Entry createEntry() {
		Entry entry = new Entry();
		entry.setSamplerClass(JDBCSampler.class);
		entry.addConfigElement(defaultSqlConfig);
		entry.addConfigElement(defaultPoolConfig);
		entry.addConfigElement(defaultDbConfig);
		return entry;
	}

}
