/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
 package org.apache.jmeter.timers.gui;

import javax.swing.*;
import javax.swing.border.*;

import java.awt.event.*;
import java.awt.*;

import org.apache.jmeter.timers.*;
import org.apache.jmeter.gui.*;
import org.apache.jmeter.util.*;

/**
 * Title:        JMeter
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Apache
 * @author Michael Stover
 * @version 1.0
 */

public class ConstantTimerGui extends JPanel implements ModelSupported,KeyListener
{
	ConstantTimer model;

	private JTextField delayField;

	private final String DELAY_FIELD = "Delay Field";

	public ConstantTimerGui()
	{
	}

	public void setModel(Object model)
	{
		this.model = (ConstantTimer)model;
		init();
	}

	public void updateGui()
	{
		delayField.setText("" + model.delay());
	}

	public static void error(Exception e, JComponent thrower) 
	{
		  JOptionPane.showMessageDialog(thrower, e, "Error", JOptionPane.ERROR_MESSAGE);
	 }

	private void init()
	{
		this.setLayout(new VerticalLayout(5, VerticalLayout.LEFT, VerticalLayout.TOP));

		// MAIN PANEL
		JPanel mainPanel = new JPanel();
		Border margin = new EmptyBorder(10, 10, 5, 10);
		mainPanel.setBorder(margin);
		mainPanel.setLayout(new VerticalLayout(5, VerticalLayout.LEFT));

		// TITLE
		JLabel panelTitleLabel = new JLabel(JMeterUtils.getResString("constant_timer_title"));
		Font curFont = panelTitleLabel.getFont();
		int curFontSize = curFont.getSize();
		curFontSize += 4;
		panelTitleLabel.setFont(new Font(curFont.getFontName(), curFont.getStyle(), curFontSize));
		mainPanel.add(panelTitleLabel);

		// NAME
		NamePanel namePanel = new NamePanel(model);
		mainPanel.add(namePanel);

		// DELAY
		JPanel delayPanel = new JPanel();
		JLabel delayLabel = new JLabel(JMeterUtils.getResString("constant_timer_delay"));
		delayPanel.add(delayLabel);
		delayField = new JTextField(6);
		delayPanel.add(delayField);
		mainPanel.add(delayPanel);

		delayField.addKeyListener(this);
		delayField.setName(DELAY_FIELD);

		this.add(mainPanel);
	}

	public void keyReleased(KeyEvent e)
		{
			String n = e.getComponent().getName();
			if (n.equals(DELAY_FIELD))
			{
				try
				{
					model.setDelay(Long.parseLong(delayField.getText()));
				}
				catch (NumberFormatException nfe)
				{
					if (delayField.getText().length() > 0)
					{
						JOptionPane.showMessageDialog(this, "You must enter a valid number",
								"Invalid data", JOptionPane.WARNING_MESSAGE);
						model.setDelay(model.getDefaultDelay());

						// We reset the text to be an empty string instead
						// of the default value. If we reset it to the
						// default value, then the user has to delete
						// that value and reenter his/her own. That's
						// too much trouble for the user. 
						delayField.setText("");
					}
					else
					{
						model.setDelay(model.getDefaultDelay());
					}
				}
			}
		}

	public void keyPressed(KeyEvent e)
	{
	}

	public void keyTyped(KeyEvent e)
	{
	}
}
